@@ -4,13 +4,14 @@
 //! Facilities for writing out TOML data from a Hakari map.
 
 use crate::hakari::{HakariBuilder, OutputMap};
-#[cfg(feature = "summaries")]
+#[cfg(feature = "cli-support")]
 use crate::summaries::HakariBuilderSummary;
 use camino::Utf8PathBuf;
 use cfg_if::cfg_if;
 use guppy::{
+    errors::TargetSpecError,
     graph::{cargo::BuildPlatform, ExternalSource, GitReq, PackageMetadata, PackageSource},
-    PackageId, TargetSpecError, Version,
+    PackageId, Version,
 };
 use std::{
     borrow::Cow,
@@ -23,14 +24,14 @@ use twox_hash::XxHash64;
 
 /// Options for Hakari TOML output.
 #[derive(Clone, Debug)]
-pub struct TomlOptions {
-    exact_versions: bool,
-    absolute_paths: bool,
-    #[cfg(feature = "summaries")]
-    builder_summary: bool,
+pub struct HakariOutputOptions {
+    pub(crate) exact_versions: bool,
+    pub(crate) absolute_paths: bool,
+    #[cfg(feature = "cli-support")]
+    pub(crate) builder_summary: bool,
 }
 
-impl TomlOptions {
+impl HakariOutputOptions {
     /// Creates a new instance with default settings.
     ///
     /// The default settings are:
@@ -40,7 +41,7 @@ impl TomlOptions {
         Self {
             exact_versions: false,
             absolute_paths: false,
-            #[cfg(feature = "summaries")]
+            #[cfg(feature = "cli-support")]
             builder_summary: false,
         }
     }
@@ -97,20 +98,20 @@ impl TomlOptions {
     /// The options are output as a header in the Hakari section:
     ///
     /// ```toml
-    /// # version = "v1"
-    /// # verify-mode = true
+    /// # resolver = "2"
+    /// # platforms = [...]
     /// ...
     /// ```
     ///
-    /// Requires the `summaries` feature to be enabled.
-    #[cfg(feature = "summaries")]
+    /// Requires the `cli-support` feature to be enabled.
+    #[cfg(feature = "cli-support")]
     pub fn set_builder_summary(&mut self, builder_summary: bool) -> &mut Self {
         self.builder_summary = builder_summary;
         self
     }
 }
 
-impl Default for TomlOptions {
+impl Default for HakariOutputOptions {
     fn default() -> Self {
         Self::new()
     }
@@ -125,8 +126,8 @@ pub enum TomlOutError {
 
     /// An error occurred while serializing TOML.
     ///
-    /// This option is only present if the `summaries` feature is enabled.
-    #[cfg(feature = "summaries")]
+    /// This option is only present if the `cli-support` feature is enabled.
+    #[cfg(feature = "cli-support")]
     Toml {
         /// A context string for the error.
         context: Cow<'static, str>,
@@ -176,7 +177,7 @@ impl fmt::Display for TomlOutError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             TomlOutError::Platform(_) => write!(f, "while serializing platform information"),
-            #[cfg(feature = "summaries")]
+            #[cfg(feature = "cli-support")]
             TomlOutError::Toml { context, .. } => write!(f, "while serializing TOML: {}", context),
             TomlOutError::FmtWrite(_) => write!(f, "while writing to fmt::Write"),
             TomlOutError::PathWithoutHakari {
@@ -200,7 +201,7 @@ impl error::Error for TomlOutError {
     fn source(&self) -> Option<&(dyn error::Error + 'static)> {
         match self {
             TomlOutError::Platform(err) => Some(err),
-            #[cfg(feature = "summaries")]
+            #[cfg(feature = "cli-support")]
             TomlOutError::Toml { err, .. } => Some(err),
             TomlOutError::FmtWrite(err) => Some(err),
             TomlOutError::PathWithoutHakari { .. } | TomlOutError::UnrecognizedExternal { .. } => {
@@ -213,11 +214,11 @@ impl error::Error for TomlOutError {
 pub(crate) fn write_toml(
     builder: &HakariBuilder<'_>,
     output_map: &OutputMap<'_>,
-    options: &TomlOptions,
+    options: &HakariOutputOptions,
     mut out: impl fmt::Write,
 ) -> Result<(), TomlOutError> {
     cfg_if::cfg_if! {
-        if #[cfg(feature = "summaries")] {
+        if #[cfg(feature = "cli-support")] {
             if options.builder_summary {
                 let summary = HakariBuilderSummary::new(builder)?;
                 summary.write_comment(&mut out)?;
@@ -244,33 +245,32 @@ pub(crate) fn write_toml(
     });
 
     for (key, vals) in output_map {
-        let cfg_str = match key.platform_idx {
-            Some(idx) => builder.platforms()[idx].triple_str(),
-            None => "'cfg(all())'",
+        let target_str = match key.platform_idx {
+            Some(idx) => format!("target.{}.", builder.platforms[idx].triple_str()),
+            None => "".to_owned(),
         };
         let dep_str = match key.build_platform {
             BuildPlatform::Target => "dependencies",
             BuildPlatform::Host => "build-dependencies",
         };
 
-        writeln!(out, "[target.{}.{}]", cfg_str, dep_str)?;
+        writeln!(out, "[{}{}]", target_str, dep_str)?;
 
         for (dep, all_features) in vals.values() {
+            let mut all_kv: Vec<Cow<str>> = Vec::with_capacity(4);
+
             // We'd ideally use serde + toml but it doesn't support inline tables. Ugh.
-            let (name, package_kv): (Cow<str>, Cow<str>) = if packages_by_name[dep.name()].len() > 1
-            {
-                (
-                    make_hashed_name(dep).into(),
-                    format!("package = \"{}\", ", dep.name()).into(),
-                )
+            let name: Cow<str> = if packages_by_name[dep.name()].len() > 1 {
+                all_kv.push(format!("package = \"{}\"", dep.name()).into());
+                make_hashed_name(dep).into()
             } else {
-                (dep.name().into(), "".into())
+                dep.name().into()
             };
 
             let source = dep.source();
             let source_kv = if source.is_crates_io() {
                 format!(
-                    "version = \"{}\", ",
+                    "version = \"{}\"",
                     VersionDisplay::new(dep.version(), options.exact_versions)
                 )
             } else {
@@ -300,9 +300,9 @@ pub(crate) fn write_toml(
                                 // TODO: is replacing \\ with / totally safe on Windows? Might run
                                 // into issues with UNC paths.
                                 let path_str = path_str.replace("\\", "/");
-                                format!("path = \"{}\", ", path_str)
+                                format!("path = \"{}\"", path_str)
                             } else {
-                                format!("path = \"{}\", ", path_str)
+                                format!("path = \"{}\"", path_str)
                             }
                         }
                     }
@@ -318,13 +318,13 @@ pub(crate) fn write_toml(
                                 repository, req, ..
                             } => {
                                 let mut out = String::new();
-                                write!(out, "git = \"{}\", ", repository)?;
+                                write!(out, "git = \"{}\"", repository)?;
                                 match req {
                                     GitReq::Branch(branch) => {
-                                        write!(out, "branch = \"{}\", ", branch)?
+                                        write!(out, ", branch = \"{}\"", branch)?
                                     }
-                                    GitReq::Tag(tag) => write!(out, "tag = \"{}\", ", tag)?,
-                                    GitReq::Rev(rev) => write!(out, "rev = \"{}\", ", rev)?,
+                                    GitReq::Tag(tag) => write!(out, ", tag = \"{}\"", tag)?,
+                                    GitReq::Rev(rev) => write!(out, ", rev = \"{}\"", rev)?,
                                     GitReq::Default => {}
                                     _ => {
                                         return Err(TomlOutError::UnrecognizedExternal {
@@ -348,22 +348,27 @@ pub(crate) fn write_toml(
                 }
             };
 
-            let default_features_kv = if all_features.contains("default") {
-                ""
-            } else {
-                "default-features = false, "
-            };
-            let all_features: Vec<_> = all_features
-                .iter()
-                .map(|feature| format!("\"{}\"", feature))
-                .collect();
-            let all_features_str = all_features.join(", ");
+            all_kv.push(source_kv.into());
 
-            writeln!(
-                out,
-                "{} = {{ {}{}{}features = [{}] }}",
-                name, package_kv, source_kv, default_features_kv, all_features_str,
-            )?;
+            if !all_features.contains("default") {
+                all_kv.push("default-features = false".into());
+            }
+
+            let features_to_write: Vec<_> = all_features
+                .iter()
+                .filter_map(|&feature| {
+                    if feature == "default" {
+                        None
+                    } else {
+                        Some(format!("\"{}\"", feature))
+                    }
+                })
+                .collect();
+            if !features_to_write.is_empty() {
+                all_kv.push(format!("features = [{}]", features_to_write.join(", ")).into());
+            };
+
+            writeln!(out, "{} = {{ {} }}", name, all_kv.join(", "))?;
         }
 
         writeln!(out)?;
